#include <my_includes.h>
#include <HypCalculator.h>
using namespace std;

complex<double> hyperg_func(const double x);
double f_s(const double q, const double R, const double alpha);
double A_1_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta);
double A_2_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta);
double A1_int(const double k, const double R, const double alpha, const double eta);
double A2_int(const double k, const double R, const double alpha, const double eta);
const double xlim1 = 0.05;
const double xlim2 = 0.95;
const int NALPHA = 6;
const int NR = 5;
const int NX1 = 50;
const int NX2 = 50;
const int NX3 = 100;
const double kmin = 2.;
const double kmin0 = 0.;
const double kmax = 100.;
const double kepsilon = 0.5;
const int NK = (int)(kmax - kmin) / kepsilon;
const double alpha_values[NALPHA] = {0.5, 0.7, 1.0, 1.3, 1.6, 1.99};
const double R_values[NR]     = {2., 3., 5., 8., 12.};
const double calc_eta(const double k, const double Mc2); // k: MeV/c, Mc2: MeV, Mc2 is the mass of the particle, not the reduced mass!!!
const complex<double> I(0., 1.);

HypCalculator* my_HypCalculator;

int main() 
{
  my_HypCalculator = new HypCalculator();
  TFile* outputfile = new TFile("A12_calc.root", "recreate");
  outputfile->cd();

  for(int iR = 0; iR < NR; iR++)
  {
    double R = R_values[iR];
    for(int ialpha = 0; ialpha < NALPHA; ialpha++)
    {
      double alpha = alpha_values[ialpha];
      TGraph* A_1 = new TGraph();
      TGraph* A_2 = new TGraph();
      TGraph* C0_k = new TGraph();
      TGraph* CG_k = new TGraph();
      TGraph* C_k = new TGraph();
      A_1->SetName(Form("A_1_graph_ialpha%d_iR%d", ialpha, iR));
      A_2->SetName(Form("A_2_graph_ialpha%d_iR%d", ialpha, iR));
      C0_k->SetName(Form("C0_k_graph_ialpha%d_iR%d", ialpha, iR));
      CG_k->SetName(Form("CG_k_graph_ialpha%d_iR%d", ialpha, iR));
      C_k->SetName(Form("C_k_graph_ialpha%d_iR%d", ialpha, iR));
      for(int ik = 0; ik < NK; ik++)
      {
        double k = kmin + ik * kepsilon;
        double k0 = kmin0 + ik * kepsilon;
        double eta = calc_eta(k, Mass_Pi);
        complex<double> a(0., eta);
        complex<double> b(1., eta);
        complex<double> c(1., 0.);
        double Gamow = (2 * M_PI * eta) / (exp(2 * M_PI * eta) - 1);
        my_HypCalculator->initialize_abc(a, b, c);
        my_HypCalculator->initialize_eta(eta);
        double A1 = A1_int(k, R, alpha, eta);
        double A2 = A2_int(k, R, alpha, eta);
        A_1->AddPoint(k, A1); 
        A_2->AddPoint(k, A2);
        C0_k->AddPoint(k0, 1 + f_s(2 * k0, R, alpha)); 
        CG_k->AddPoint(k, Gamow * (1 + f_s(2 * k, R, alpha))); 
        C_k->AddPoint(k, Gamow * (1 + f_s(2 * k, R, alpha) + (eta / M_PI) * (A1 + A2))); 
//        cout << "    ik = " << ik << " finished." << endl;
      }
      A_1->Write();
      A_2->Write();
      C0_k->Write();
      CG_k->Write();
      C_k->Write();
      delete A_1;
      delete A_2;
      delete C0_k;
      delete CG_k;
      delete C_k;
      cout << "    ialpha = " << ialpha << " finished." << endl;
    }
    cout << "    iR = " << iR << " finished." << endl;
  }

  outputfile->Close();
  delete outputfile;
  delete my_HypCalculator;
  return 0;
}

complex<double> hyperg_func(const double x) 
{
  return my_HypCalculator->F1_F2_F3(x);
}

double f_s(const double q, const double R, const double alpha)
{
  return exp(-0.5 * pow(fabs(q*R / HBARC), alpha));
}

double A_1_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta)
{
  double fs1_1 = (f_s(2.*k*x, R, alpha) - f_s(0., R, alpha)) / x;
  double fs1_2 = (f_s(2.*k/x, R, alpha) - f_s(0., R, alpha)) / x;
  complex<double> func_1 = pow(1. + 1./x, 2.*eta*I) * hyperg_func(1. / (x*x));
  complex<double> func_2 = pow(1. + x, 2.*eta*I) * hyperg_func(x*x);
  return -2./eta * imag(fs1_1 * func_1 + fs1_2 * func_2);
}

double A_2_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta)
{
 double func_1 = sin(eta * log((1. + x)/(1. - x))) / (x * (x + 1.));
 double func_2 = x * x * exp(M_PI * eta) * (f_s(2. * k * x, R, alpha) - f_s(2. * k, R, alpha)) / (1. - x);
 double func_3 = (f_s(2. * k / x, R, alpha) - f_s(2. * k, R, alpha)) / (1. - x);
 return (2. / eta ) * (func_1 * func_2 - func_1 * func_3);
}

const double calc_eta(const double k, const double Mc2)
{
  return FINESTRUCTURE_CONSTANT * Mc2 * 0.5 / k;
}

double A1_int(const double k, const double R, const double alpha, const double eta)
{
  double sum = 0.;
  double epsilon1 = xlim1 / NX1;
  double epsilon2 = (xlim2 - xlim1) / NX2;
  double epsilon3 = (1. - xlim2) / NX3;
  for(int ipoint = 0; ipoint < NX1; ipoint++)
    sum += A_1_s_wo_int(epsilon1 * (ipoint + 0.5), k, R, alpha, eta) * epsilon1;
  for(int ipoint = 0; ipoint < NX2; ipoint++)
    sum += A_1_s_wo_int(xlim1 + (epsilon2 * (ipoint + 0.5)), k, R, alpha, eta) * epsilon2;
  for(int ipoint = 0; ipoint < NX3; ipoint++)
    sum += A_1_s_wo_int(xlim2 + (epsilon3 * (ipoint + 0.5)), k, R, alpha, eta) * epsilon3;
  return sum;
}

double A2_int(const double k, const double R, const double alpha, const double eta)
{
  double sum = 0;
  double epsilon1 = xlim1 / NX1;
  double epsilon2 = (xlim2 - xlim1) / NX2;
  double epsilon3 = (1. - xlim2) / NX3;
  for(int ipoint = 0; ipoint < NX1; ipoint++)
    sum += A_2_s_wo_int(epsilon1 * (ipoint + 0.5), k, R, alpha, eta) * epsilon1;
  for(int ipoint = 0; ipoint < NX2; ipoint++)
    sum += A_2_s_wo_int(xlim1 + (epsilon2 * (ipoint + 0.5)), k, R, alpha, eta) * epsilon2;
  for(int ipoint = 0; ipoint < NX3; ipoint++)
    sum += A_2_s_wo_int(xlim2 + (epsilon3 * (ipoint + 0.5)), k, R, alpha, eta) * epsilon3;
  return sum;
}



