#include <my_includes.h>
#include <HypCalculator.h>
using namespace std;

complex<double> hyperg_func(const double x);
double f_s(const double q, const double R, const double alpha);
double A_1_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta);
double A_2_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta);
const int NK = 8;
const int NALPHA = 6;
const int NR = 5;
const double k_values[NK]     = {2., 4., 6., 10., 15., 20., 40., 80.};
const double alpha_values[NK] = {0.5, 0.7, 1.0, 1.3, 1.6, 1.99};
const double R_values[NK]     = {2., 3., 5., 8., 12.};
const int npoints = 10000;
const double calc_eta(const double k, const double Mc2); // k: MeV/c, Mc2: MeV, Mc2 is the mass of the particle, not the reduced mass!!!
const complex<double> I(0., 1.);

HypCalculator* my_HypCalculator;

int main() 
{
  my_HypCalculator = new HypCalculator();
  TFile* outputfile = new TFile("A_s_calc.root", "recreate");
  outputfile->cd();

  for(int ik = 0; ik < NK; ik++)
  {
    for(int ialpha = 0; ialpha < NALPHA; ialpha++)
    {
      for(int iR = 0; iR < NR; iR++)
      {
        double k = k_values[ik];
        double alpha = alpha_values[ialpha];
        double R = R_values[iR];

        double eta = calc_eta(k, Mass_Pi);
        complex<double> a(0., eta);
        complex<double> b(1., eta);
        complex<double> c(1., 0.);
        my_HypCalculator->initialize_abc(a, b, c);
        my_HypCalculator->initialize_eta(eta);
        TGraph* A_1 = new TGraph();
        TGraph* A_2 = new TGraph();
        for(int ipoint = 0; ipoint < npoints; ipoint++)
        {
           double x = 1. / npoints * (ipoint + .5);
           A_1->AddPoint(x, A_1_s_wo_int(x, k, R, alpha, eta));
           A_2->AddPoint(x, A_2_s_wo_int(x, k, R, alpha, eta));
        }
        stringstream A_1_name(""); A_1_name << "A_1_graph_ik" << ik << "_ia" << ialpha << "_iR" << iR; 
        stringstream A_2_name(""); A_2_name << "A_2_graph_ik" << ik << "_ia" << ialpha << "_iR" << iR; 
        A_1->SetName(Form("A_1_graph_ik%d_ialpha%d_iR%d", ik, ialpha, iR));
        A_2->SetName(Form("A_2_graph_ik%d_ialpha%d_iR%d", ik, ialpha, iR));
        A_1->Write();
        A_2->Write();
        delete A_1;
        delete A_2;
        cout << "    iR = " << iR << " finished." << endl;
      }
      cout << "    ialpha = " << ialpha << " finished." << endl;
    }
    cout << "    ik = " << ik << " finished." << endl;
  }

  outputfile->Close();
  delete outputfile;
  delete my_HypCalculator;
  return 0;
}

complex<double> hyperg_func(const double x) 
{
  return my_HypCalculator->F1_F2_F3(x);
}

double f_s(const double q, const double R, const double alpha)
{
  return exp(-0.5 * pow(fabs(q*R / HBARC), alpha));
}

double A_1_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta)
{
  double fs1_1 = (f_s(2.*k*x, R, alpha) - f_s(0., R, alpha)) / x;
  double fs1_2 = (f_s(2.*k/x, R, alpha) - f_s(0., R, alpha)) / x;
  complex<double> func_1 = pow(1. + 1./x, 2.*eta*I) * hyperg_func(1. / (x*x));
  complex<double> func_2 = pow(1. + x, 2.*eta*I) * hyperg_func(x*x);
  return -2./eta * imag(fs1_1 * func_1 + fs1_2 * func_2);
}

double A_2_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta)
{
 double func_1 = sin(eta * log((1. + x)/(1. - x))) / (x * (x + 1.));
 double func_2 = x * x * exp(M_PI * eta) * (f_s(2. * k * x, R, alpha) - f_s(2. * k, R, alpha)) / (1. - x);
 double func_3 = (f_s(2. * k / x, R, alpha) - f_s(2. * k, R, alpha)) / (1. - x);
 return (2. / eta ) * (func_1 * func_2 - func_1 * func_3);
}

const double calc_eta(const double k, const double Mc2)
{
  return FINESTRUCTURE_CONSTANT * Mc2 * 0.5 / k;
}





