#include <my_includes.h>
using namespace std;
const int colors[10] = {2, 3, 4, 6, 7, 28, 38, 40, 41, 46};

const char* infilename = "/home/aletta/work/A_s_calc.root";
const char* figpath = "/home/aletta/work/figs/A_test/";

const int NK = 8;
const int NALPHA = 6;
const int NR = 5;
const double k_values[NK]     = {2., 4., 6., 10., 15., 20., 40., 80.};
const double alpha_values[NK] = {0.5, 0.7, 1.0, 1.3, 1.6, 1.99};
const double R_values[NK]     = {2., 3., 5., 8., 12.};

int main() 
{
  TCanvas* c1 = new TCanvas("c1", "Canvas", 10, 10, 1400, 1000);
  c1->SetLeftMargin(0.05);
  c1->SetRightMargin(0.15);
  c1->SetTopMargin(0.08);
  c1->SetBottomMargin(0.08);
  TFile* infile = new TFile(infilename, "read");
  for(int ialpha = 0; ialpha < NALPHA; ialpha++)
  {
    for(int iR = 0; iR < NR; iR++)
    {
      TGraph* A_1[NK];
      TGraph* A_2[NK];
      TLegend* leg_1 = new TLegend(0.87, 0.4, 0.99, 0.88); leg_1->SetFillColor(0); leg_1->SetBorderSize(0);
      TLegend* leg_2 = new TLegend(0.87, 0.4, 0.99, 0.88); leg_2->SetFillColor(0); leg_2->SetBorderSize(0);
      stringstream ttl1(""); ttl1 << "A_{1} integrand vs. x, #alpha = " << alpha_values[ialpha] << ", R = " << R_values[iR] << ";x";
      stringstream ttl2(""); ttl2 << "A_{2} integrand vs. x, #alpha = " << alpha_values[ialpha] << ", R = " << R_values[iR] << ";x";
      for(int ik = 0; ik < NK; ik++)
      {
        double k = k_values[ik];
        double alpha = alpha_values[ik];
        double R = R_values[ik];

        A_1[ik] = (TGraph*)infile->Get(Form("A_1_graph_ik%d_ialpha%d_iR%d", ik, ialpha, iR));
        A_2[ik] = (TGraph*)infile->Get(Form("A_2_graph_ik%d_ialpha%d_iR%d", ik, ialpha, iR));
        A_1[ik]->SetLineColor(colors[ik]);
        A_2[ik]->SetLineColor(colors[ik]);
        A_1[ik]->SetLineWidth(2.);
        A_2[ik]->SetLineWidth(2.);
        if(ik == 0)
        {
          A_1[ik]->SetTitle(ttl1.str().c_str());
          A_2[ik]->SetTitle(ttl2.str().c_str());
          A_1[ik]->GetXaxis()->SetRangeUser(0.0, 1.);
          A_2[ik]->GetXaxis()->SetRangeUser(0.0, 1.);
          A_1[ik]->GetYaxis()->SetRangeUser(-5., 30.);
          A_2[ik]->GetYaxis()->SetRangeUser(-5., 10.);
        }
        A_2[ik]->SetLineColor(colors[ik]);
        leg_1->AddEntry(A_1[ik], Form("k = %g", k_values[ik]), "l");
        leg_2->AddEntry(A_2[ik], Form("k = %g", k_values[ik]), "l");
      }

      for(int ik = 0; ik < NK; ik++)
        A_1[ik]->Draw(ik == 0 ? "" : "same");
      leg_1->Draw("same");
      c1->Print(Form("%sA_1_ialpha%d_iR%d.png", figpath, ialpha, iR));
      c1->Clear();

      for(int ik = 0; ik < NK; ik++)
        A_2[ik]->Draw(ik == 0 ? "" : "same");
      leg_2->Draw("same");
      c1->Print(Form("%sA_2_ialpha%d_iR%d.png", figpath, ialpha, iR));
      c1->Clear();

      for(int ik = 0; ik < NK; ik++)
      {
        delete A_1[ik];
        delete A_2[ik];
      }
    }
  }
  delete infile;
  delete c1;
  return 0;
}
