#include <my_includes.h>
using namespace std;
const int colors[10] = {2, 3, 4, 6, 7, 28, 38, 40, 41, 46};
double calc_eta(const double k, const double Mc2);
double f_s(const double q, const double R, const double alpha);

const char* infilename = "/home/aletta/work/A12_calc.root";
const char* figpath = "/home/aletta/work/figs/A_int/";
const int NALPHA = 6;
const int NR = 5;
const double alpha_values[NALPHA] = {0.5, 0.7, 1.0, 1.3, 1.6, 1.99};
const double R_values[NR]     = {2., 3., 5., 8., 12.};
const double kmin = 2.;
const double kmin0 = 0.;
const double kmax = 100.;
const double kepsilon = 0.5;
const int NK = (int)(kmax - kmin) / kepsilon;

int main() 
{
  TCanvas* c1 = new TCanvas("c1", "Canvas", 10, 10, 1400, 1000);
  c1->SetLeftMargin(0.10);
  c1->SetRightMargin(0.17);
  c1->SetTopMargin(0.09);
  c1->SetBottomMargin(0.12);
  TFile* infile = new TFile(infilename, "read");
  TGraph* line = new TGraph();
  for(int ik = 0; ik < NK; ik++)
  {
    line->AddPoint(kmin + ik * kepsilon, 1.);
  }
  line->SetLineStyle(kDashed);
  line->SetLineWidth(2.);
//  line->SetLineColor(38);
  for(int iR = 0; iR < NR; iR++)
  {
    TMultiGraph *mg_A1 = new TMultiGraph();
    TMultiGraph *mg_A2 = new TMultiGraph();
    TMultiGraph *mg_C0 = new TMultiGraph();
    TMultiGraph *mg_CG = new TMultiGraph();
    TMultiGraph *mg_Ck = new TMultiGraph();
    TLegend* leg_A1 = new TLegend(0.84, 0.4, 0.99, 0.88); leg_A1->SetFillColor(0); leg_A1->SetBorderSize(0);
    TLegend* leg_A2 = new TLegend(0.84, 0.4, 0.99, 0.88); leg_A2->SetFillColor(0); leg_A2->SetBorderSize(0);
    TLegend* leg_C0 = new TLegend(0.84, 0.4, 0.99, 0.88); leg_A2->SetFillColor(0); leg_A2->SetBorderSize(0);
    TLegend* leg_CG = new TLegend(0.84, 0.4, 0.99, 0.88); leg_A2->SetFillColor(0); leg_A2->SetBorderSize(0);
    TLegend* leg_Ck = new TLegend(0.84, 0.4, 0.99, 0.88); leg_A2->SetFillColor(0); leg_A2->SetBorderSize(0);
    double R = R_values[iR];
    mg_A1->SetTitle(Form("Results of the A_{1} integral (R = %g fm);k [MeV/c];A_{1}", R));
    mg_A2->SetTitle(Form("Results of the A_{2} integral (R = %g fm);k [MeV/c];A_{1}", R));
    mg_C0->SetTitle(Form("C^{0}(k) correlation function (R = %g fm);k [MeV/c];C^{0}(k)", R));
    mg_CG->SetTitle(Form("C^{G}(k) correlation function (R = %g fm);k [MeV/c];C^{G}(k)", R));
    mg_Ck->SetTitle(Form("C(k) correlation function (R = %g fm);k [MeV/c];C(k)", R));
    TGraph* A_1[NALPHA];
    TGraph* A_2[NALPHA];
    TGraph* C0[NALPHA];
    TGraph* CG[NALPHA];
    TGraph* Ck[NALPHA];
    for(int ialpha = 0; ialpha < NALPHA; ialpha++)
    {
      double alpha = alpha_values[ialpha];
      A_1[ialpha] = (TGraph*)infile->Get(Form("A_1_graph_ialpha%d_iR%d", ialpha, iR));
      A_2[ialpha] = (TGraph*)infile->Get(Form("A_2_graph_ialpha%d_iR%d", ialpha, iR));
      C0[ialpha] = (TGraph*)infile->Get(Form("C0_k_graph_ialpha%d_iR%d", ialpha, iR));
      CG[ialpha] = (TGraph*)infile->Get(Form("CG_k_graph_ialpha%d_iR%d", ialpha, iR));
      Ck[ialpha] = (TGraph*)infile->Get(Form("C_k_graph_ialpha%d_iR%d", ialpha, iR));
      A_1[ialpha]->SetLineColor(colors[ialpha]);
      A_2[ialpha]->SetLineColor(colors[ialpha]);
      C0[ialpha]->SetLineColor(colors[ialpha]);
      CG[ialpha]->SetLineColor(colors[ialpha]);
      Ck[ialpha]->SetLineColor(colors[ialpha]);
      A_1[ialpha]->SetLineWidth(2.);
      A_2[ialpha]->SetLineWidth(2.);
      C0[ialpha]->SetLineWidth(2.);
      CG[ialpha]->SetLineWidth(2.);
      Ck[ialpha]->SetLineWidth(2.);
      mg_A1->Add(A_1[ialpha]);
      mg_A2->Add(A_2[ialpha]);
      mg_C0->Add(C0[ialpha]);
      mg_CG->Add(CG[ialpha]);
      mg_Ck->Add(Ck[ialpha]);
      leg_A1->AddEntry(A_1[ialpha], Form("#alpha = %g", alpha_values[ialpha]), "l");
      leg_A2->AddEntry(A_2[ialpha], Form("#alpha = %g", alpha_values[ialpha]), "l");
      leg_C0->AddEntry(A_2[ialpha], Form("#alpha = %g", alpha_values[ialpha]), "l");
      leg_CG->AddEntry(A_2[ialpha], Form("#alpha = %g", alpha_values[ialpha]), "l");
      leg_Ck->AddEntry(A_2[ialpha], Form("#alpha = %g", alpha_values[ialpha]), "l");
    }

    mg_A1->GetXaxis()->SetRangeUser(0., 100.);
    mg_A1->GetYaxis()->SetRangeUser(0., 10.);
    mg_A1->Draw("a");
    leg_A1->Draw("same");
    c1->Print(Form("%sA_1_iR%d.png", figpath, iR));
    c1->Clear();

    mg_A2->GetXaxis()->SetRangeUser(0., 100.);
    mg_A2->GetYaxis()->SetRangeUser(0., 4.);
    mg_A2->Draw("a");
    leg_A2->Draw("same");
    c1->Print(Form("%sA_2_iR%d.png", figpath, iR));
    c1->Clear();

    mg_C0->GetXaxis()->SetRangeUser(0., 100.);
    mg_C0->GetYaxis()->SetRangeUser(0., 2.2);
    mg_C0->Draw("a");
    line->Draw("same");
    leg_C0->Draw("same");
    c1->Print(Form("%sC_0_iR%d.png", figpath, iR));
    c1->Clear();

    mg_CG->GetXaxis()->SetRangeUser(0., 100.);
    mg_CG->GetYaxis()->SetRangeUser(0., 2.2);
    mg_CG->Draw("a");
    line->Draw("same");
    leg_CG->Draw("same");
    c1->Print(Form("%sC_G_iR%d.png", figpath, iR));
    c1->Clear();

    mg_Ck->GetXaxis()->SetRangeUser(0., 100.);
    mg_Ck->GetYaxis()->SetRangeUser(0., 2.2);
    mg_Ck->Draw("a");
    line->Draw("same");
    leg_Ck->Draw("same");
    c1->Print(Form("%sC_k_iR%d.png", figpath, iR));
    c1->Clear();

//    for(int ialpha = 0; ialpha < NALPHA; ialpha++)
//    {
//      delete A_1[ialpha];
//      delete A_2[ialpha];
//    }
    delete mg_A1;
    delete mg_A2;
    delete mg_C0;
    delete mg_CG;
    delete mg_Ck;
  }
  delete infile;
  delete c1;
  return 0;
}

double calc_eta(const double k, const double Mc2)
{
  return FINESTRUCTURE_CONSTANT * Mc2 * 0.5 / k;
}

double f_s(const double q, const double R, const double alpha)
{
  return exp(-0.5 * pow(fabs(q*R / HBARC), alpha));
}

//double A_1_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta)
