#include <my_includes.h>
#include <HypCalculator.h>
using namespace std;

const double pi = 3.1415926539;
complex<double> hyperg_func(const double x);
double f_s(const double q, const double R, const double alpha);
double A_2_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta);
//double A_1_s_wo_int(const double q, const double k, const double R, const double alpha, const double eta);
double A_1_s_wo_int(const double q, const double k, const double R, const double alpha, const double eta);
const int eta_points = 5;
const double eta = 0.1;
const double epsilon = 1e-12;
const double xmin1 = 0.;
const double xmax1 = 60.;
const double xmin2 = 0.;
const double xmax2 = 1.;
const double alpha = 1.5;
const double R = 1.;
const double k = 10.;
const int npoints = 10000;

HypCalculator* my_HypCalculator;

int main() 
{
  my_HypCalculator = new HypCalculator();

  TFile* outputfile = new TFile("A_s_calc.root", "recreate");
  outputfile->cd();
  double x1[npoints];
  double x2[npoints];
  complex<double> a(0., eta);
  complex<double> b(1., eta);
  complex<double> c(1., 0.);
  my_HypCalculator->initialize_abc(a, b, c);
  my_HypCalculator->initialize_eta(eta);

  TGraph* A_1 = new TGraph();
  TGraph* A_2 = new TGraph();
  for(int ipoint = 0; ipoint < npoints; ipoint++)
  {
     x1[ipoint] = xmin1 + (xmax1 - xmin1) * 1.0 / npoints * (ipoint + 0.5);
     x2[ipoint] = xmin2 + (xmax2 - xmin2) * 1.0 / npoints * (ipoint + 0.5);
     A_1->AddPoint(x1[ipoint], A_1_s_wo_int(x1[ipoint], k, R, alpha, eta));
     A_2->AddPoint(x2[ipoint], A_2_s_wo_int(x2[ipoint], k, R, alpha, eta));
  }

  A_1->SetName("A_1_graph");
  A_2->SetName("A_2_graph");
  A_1->Write();
  A_2->Write();
  outputfile->Close();
  delete outputfile;
  delete A_1;
  delete A_2;
  delete my_HypCalculator;
  return 0;
}

complex<double> hyperg_func(const double x) 
{
  return my_HypCalculator->F1_F2_F3(x);
}

double f_s(const double q, const double R, const double alpha)
{
  return exp(-0.5 * pow(fabs(q*R), alpha));
}

double A_2_s_wo_int(const double x, const double k, const double R, const double alpha, const double eta)
{
 double func_1 = sin(eta * log((1. + x)/(1. - x))) / (x * (x + 1.));
 double func_2 = x * x * exp(pi * eta) * (f_s(2. * k * x, R, alpha) - f_s(2. * k, R, alpha)) / (1. - x);
 double func_3 = (f_s(2. * k / x, R, alpha) - f_s(2. * k, R, alpha)) / (1. - x);
 return (2. / eta ) * (func_1 * func_2 - func_1 * func_3);
}

double A_1_s_wo_int(const double q, const double k, const double R, const double alpha, const double eta)
{
  complex<double> I(0., 1.);
  double func_1 = (f_s(q, R, alpha) - f_s(0., R, alpha)) / q;
  complex<double> func_2 = pow(1. + ((2. * k) / (q)), 2* eta * I) * hyperg_func(4 * k * k / (q * q));
  return - (2 / eta) * func_1 * imag(func_2);
}







