#include <my_includes.h>
using namespace std;

complex<double> hyperg_func(double eta, double z);
const int eta_points = 5;
const double eta[eta_points] = {0., 0.01, 0.1, 0.5, 1.};
double epsilon = 1e-12;
double xmin = -0.9;
double xmax = 0.9;
const int npoints = 10000;

int main() 
{
  double x[npoints];
  double F_real[eta_points][npoints];
  double F_imag[eta_points][npoints];

  for(int ipoint = 0; ipoint < npoints; ipoint++)
    x[ipoint] = xmin + (xmax - xmin) * 1.0 / npoints * (ipoint + 0.5);

  for(int ipoint = 0; ipoint < npoints; ipoint++)
  {
    for(int eta_ipoint = 0; eta_ipoint < eta_points; eta_ipoint++)
    { 
      F_real[eta_ipoint][ipoint] = real(hyperg_func(eta[eta_ipoint], x[ipoint]));	
      F_imag[eta_ipoint][ipoint] = imag(hyperg_func(eta[eta_ipoint], x[ipoint]));	
    }
  }
  TFile* outputfile = new TFile("hyperg_graphs.root", "recreate");
  TGraph* g_real[eta_points];
  TGraph* g_imag[eta_points];
  outputfile->cd();
  for(int ieta = 0; ieta < eta_points; ieta++)
  { 
    g_real[ieta] = new TGraph();
    g_imag[ieta] = new TGraph();
    for (int ipoint = 0; ipoint < npoints; ipoint++)
    {
      g_real[ieta]->AddPoint(x[ipoint], F_real[ieta][ipoint]); 
      g_imag[ieta]->AddPoint(x[ipoint], F_imag[ieta][ipoint]); 
    }
    g_real[ieta]->SetName(Form("gr_real%d", ieta+1));
    g_imag[ieta]->SetName(Form("gr_imag%d", ieta+1));
    g_real[ieta]->Write();
    g_imag[ieta]->Write();
  }
  outputfile->Close();
  for (int ieta = 0; ieta < eta_points; ieta++)
  {
    delete g_real[ieta];
    delete g_imag[ieta];
  }
  delete outputfile;
  return 0;
}

complex<double> hyperg_func(double eta, double z) 
{
  complex<double> n(0., 0.);
  complex<double> sum(1., 0);
  complex<double> element(1., 0);
  complex<double> i(0.,1.);
  complex<double> a = i*eta;
  complex<double> b = 1. + a;
  complex<double> c = 1.;
  while(abs(element) > epsilon)
  {
    element = element * (z/(n + 1.)) * (a + n) * (b + n) / (c + n);
    sum += element;
    n += 1. ;
  }
  return sum;
}

