#include <my_includes.h>
using namespace std;

double hyperg_func(double z, double a, int& nterms);
double epsilon = 1e-16;
double xmin = -0.99;
double xmax = 0.99;
double a = 2.;
const int npoints = 10000;

int main() 
{
  double x[npoints];
  double F[npoints];
  int nterm_values[npoints];

  for(int ipoint = 0; ipoint < npoints; ipoint++)
    x[ipoint] = xmin + (xmax - xmin) * 1.0 / npoints * (ipoint + 0.5);

  for(int ipoint = 0; ipoint < npoints; ipoint++)
  {
      F[ipoint] = hyperg_func(x[ipoint], a, nterm_values[ipoint]);	
  }
  TCanvas* c1 = new TCanvas("c1", "Canvas", 10, 10, 700, 500);
//  TGraph *gr = new TGraph(npoints, x, F);
  TF1* func = new TF1("function", "pow((1-x),-[0])", xmin, xmax);
  func->SetParameter(0,a);
//  gr->SetLineColor(619);
//  func->SetLineColor(400);
//  gr->SetMarkerColor(619);
//  gr->SetLineWidth(2.5);
//  gr->SetTitle("Real part of hypergeometric function; x; F");
//  gr->Draw("");
//  func->Draw("same");

  TGraph* gr_diff = new TGraph();
  TGraph* gr_nterms = new TGraph();
  double function_value;
  for(int ipoint = 0; ipoint < npoints; ipoint++)
  { 
    function_value = func->Eval(x[ipoint]);
    gr_diff->AddPoint(x[ipoint], fabs(function_value-F[ipoint])/function_value);
    gr_nterms->AddPoint(x[ipoint], nterm_values[ipoint]);
  }
  gr_diff->SetLineColor(882);
  gr_diff->SetMarkerColor(882);
  gr_diff->SetLineWidth(2);
  gr_diff->SetTitle("Relative errors;x;difference");
  gr_diff->Draw("");
  c1->Print("hypergeometric_check.png");
  c1->Clear();

  gr_nterms->SetLineColor(4);
  gr_nterms->SetMarkerColor(4);
  gr_nterms->SetLineWidth(2);
  gr_nterms->SetTitle("#of terms in hyp.geom check;x;nterms");
  gr_nterms->Draw("");
  c1->Print("hypergeometric_check_nterms.png");
  c1->Clear();

//  c1->Print(figname2);
//
//  leg->AddEntry(gr_diff_gauss, "#alpha = 2", "l");
//  leg->AddEntry(gr_diff_lorenz, "#alpha = 1", "l");
//  mg->Add(gr_diff_gauss);
//  mg->Add(gr_diff_lorenz);
//  mg->SetTitle("Relative errors of the numerical integrations;r;difference");
//  mg->Draw("a");
//  leg->Draw();
//  c1->Print(figname1);
//
  delete gr_diff;
  delete gr_nterms;
  delete c1;
  return 0;
}

double hyperg_func(double z, double a, int& nterms) 
{
  double n = 0.;
  double sum = 1.;
  double element = 1.;
  double b = 1.;
  double c = 1.;
  while(fabs(element) > epsilon)
  {
    element = element * (z/(n + 1.)) * (a + n) * (b + n) / (c + n);
    sum += element;
    n += 1. ;
  }
  nterms = (int)(n+0.01);
  return sum;
}

