#include <my_includes.h>
using namespace std;
const double pi = 3.14159265358979323846;

double Levy_reduced(const double alpha, const double rred);
const double NsinPoints= 25.; // #points on quarter period of sine
const double NexpPoints = 25.; // #points on exponential decay
const double expLimit = 1e-20; 

const double rmin = 0.0;
const double rmax = 4.0;
const int npoints = 200;
const int alpha_points = 5;
const double alpha[alpha_points] = {2.0, 1.7, 1.5, 1.2, 1.};

int main() 
{
  double rred[npoints];
  double F[alpha_points][npoints];

  for(int ipoint = 0; ipoint < npoints; ipoint++)
    rred[ipoint] = rmin + (rmax - rmin) * 1.0 / npoints * (ipoint + 0.5);

  for(int ipoint = 0; ipoint < npoints; ipoint++)
  {
    for(int alpha_ipoint = 0; alpha_ipoint < alpha_points; alpha_ipoint++)
    { 
      F[alpha_ipoint][ipoint] = Levy_reduced(alpha[alpha_ipoint], rred[ipoint]);	
    }
  }
  TFile* outputfile = new TFile("levy_graphs.root", "recreate");
  TGraph* g[alpha_points];
  outputfile->cd();
  for(int ialpha = 0; ialpha < alpha_points; ialpha++)
  { 
    g[ialpha] = new TGraph();
    for (int ipoint = 0; ipoint < npoints; ipoint++)
    {
      g[ialpha]->AddPoint(rred[ipoint], F[ialpha][ipoint]); 
    }
    g[ialpha]->SetName(Form("gr%d", ialpha+1));
    g[ialpha]->Write();
  }
  outputfile->Close();
  for (int ialpha = 0; ialpha < alpha_points; ialpha++)
  {
    delete g[ialpha];
  }
  delete outputfile;
  return 0;
}

double Levy_reduced(const double alpha, const double rred)
{
  double r = rred;
  double qmax = pow(-2.*log(expLimit), 1./alpha); // so that exp(-0.5*pow(qmax, alpha)) < expLimit
  double epsilon_expdecay = pow(2.0, 1./alpha) / NexpPoints; // so that the exponential decay contains NexpPoints points at least.
  double epsilon_sinperiod = (2. * pi / 4.) / NsinPoints / (r + 0.001); // so that at least 4*NsinPoints points fit on a whole period of the sine function
  double epsilon = epsilon_expdecay;
  if(epsilon_sinperiod < epsilon)
    epsilon = epsilon_sinperiod;
  double q = epsilon / 2.0;
  double integral_value = 0.0;

  while(q < qmax)
  {
    integral_value += q * sin(q * r) *exp( -0.5 * pow(q, alpha));
    q += epsilon;
  } 

  return integral_value * epsilon / ( 2.0 * pi * pi * r );
}

